#!/usr/bin/python
from __future__ import absolute_import, division, print_function
# Copyright 2019-2021 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fmgr_system_autodelete
short_description: Automatic deletion policy for logs, reports, archived, and quarantined files.
description:
    - This module is able to configure a FortiManager device.
    - Examples include all parameters and values which need to be adjusted to data sources before usage.

version_added: "2.10"
author:
    - Link Zheng (@chillancezen)
    - Jie Xue (@JieX19)
    - Frank Shen (@fshen01)
    - Hongbin Lu (@fgtdev-hblu)
notes:
    - Running in workspace locking mode is supported in this FortiManager module, the top
      level parameters workspace_locking_adom and workspace_locking_timeout help do the work.
    - To create or update an object, use state present directive.
    - To delete an object, use state absent directive.
    - Normally, running one module can fail when a non-zero rc is returned. you can also override
      the conditions to fail or succeed with parameters rc_failed and rc_succeeded

options:
    enable_log:
        description: Enable/Disable logging for task
        required: false
        type: bool
        default: false
    proposed_method:
        description: The overridden method for the underlying Json RPC request
        required: false
        type: str
        choices:
          - update
          - set
          - add
    bypass_validation:
        description: only set to True when module schema diffs with FortiManager API structure, module continues to execute without validating parameters
        required: false
        type: bool
        default: false
    workspace_locking_adom:
        description: the adom to lock for FortiManager running in workspace mode, the value can be global and others including root
        required: false
        type: str
    workspace_locking_timeout:
        description: the maximum time in seconds to wait for other user to release the workspace lock
        required: false
        type: int
        default: 300
    state:
        description: the directive to create, update or delete an object
        type: str
        required: true
        choices:
          - present
          - absent
    rc_succeeded:
        description: the rc codes list with which the conditions to succeed will be overriden
        type: list
        required: false
    rc_failed:
        description: the rc codes list with which the conditions to fail will be overriden
        type: list
        required: false
    system_autodelete:
        description: the top level parameters set
        required: false
        type: dict
        suboptions:
            dlp-files-auto-deletion:
                description: no description
                type: dict
                required: false
                suboptions:
                    retention:
                        type: str
                        default: 'days'
                        description:
                         - 'Automatic deletion in days, weeks, or months.'
                         - 'days - Auto-delete data older than <value> days.'
                         - 'weeks - Auto-delete data older than <value> weeks.'
                         - 'months - Auto-delete data older than <value> months.'
                        choices:
                            - 'days'
                            - 'weeks'
                            - 'months'
                    runat:
                        type: int
                        default: 0
                        description: 'Automatic deletion run at (0 - 23) oclock.'
                    status:
                        type: str
                        default: 'disable'
                        description:
                         - 'Enable/disable automatic deletion.'
                         - 'disable - Disable automatic deletion.'
                         - 'enable - Enable automatic deletion.'
                        choices:
                            - 'disable'
                            - 'enable'
                    value:
                        type: int
                        default: 0
                        description: 'Automatic deletion in x days, weeks, or months.'
            log-auto-deletion:
                description: no description
                type: dict
                required: false
                suboptions:
                    retention:
                        type: str
                        default: 'days'
                        description:
                         - 'Automatic deletion in days, weeks, or months.'
                         - 'days - Auto-delete data older than <value> days.'
                         - 'weeks - Auto-delete data older than <value> weeks.'
                         - 'months - Auto-delete data older than <value> months.'
                        choices:
                            - 'days'
                            - 'weeks'
                            - 'months'
                    runat:
                        type: int
                        default: 0
                        description: 'Automatic deletion run at (0 - 23) oclock.'
                    status:
                        type: str
                        default: 'disable'
                        description:
                         - 'Enable/disable automatic deletion.'
                         - 'disable - Disable automatic deletion.'
                         - 'enable - Enable automatic deletion.'
                        choices:
                            - 'disable'
                            - 'enable'
                    value:
                        type: int
                        default: 0
                        description: 'Automatic deletion in x days, weeks, or months.'
            quarantine-files-auto-deletion:
                description: no description
                type: dict
                required: false
                suboptions:
                    retention:
                        type: str
                        default: 'days'
                        description:
                         - 'Automatic deletion in days, weeks, or months.'
                         - 'days - Auto-delete data older than <value> days.'
                         - 'weeks - Auto-delete data older than <value> weeks.'
                         - 'months - Auto-delete data older than <value> months.'
                        choices:
                            - 'days'
                            - 'weeks'
                            - 'months'
                    runat:
                        type: int
                        default: 0
                        description: 'Automatic deletion run at (0 - 23) oclock.'
                    status:
                        type: str
                        default: 'disable'
                        description:
                         - 'Enable/disable automatic deletion.'
                         - 'disable - Disable automatic deletion.'
                         - 'enable - Enable automatic deletion.'
                        choices:
                            - 'disable'
                            - 'enable'
                    value:
                        type: int
                        default: 0
                        description: 'Automatic deletion in x days, weeks, or months.'
            report-auto-deletion:
                description: no description
                type: dict
                required: false
                suboptions:
                    retention:
                        type: str
                        default: 'days'
                        description:
                         - 'Automatic deletion in days, weeks, or months.'
                         - 'days - Auto-delete data older than <value> days.'
                         - 'weeks - Auto-delete data older than <value> weeks.'
                         - 'months - Auto-delete data older than <value> months.'
                        choices:
                            - 'days'
                            - 'weeks'
                            - 'months'
                    runat:
                        type: int
                        default: 0
                        description: 'Automatic deletion run at (0 - 23) oclock.'
                    status:
                        type: str
                        default: 'disable'
                        description:
                         - 'Enable/disable automatic deletion.'
                         - 'disable - Disable automatic deletion.'
                         - 'enable - Enable automatic deletion.'
                        choices:
                            - 'disable'
                            - 'enable'
                    value:
                        type: int
                        default: 0
                        description: 'Automatic deletion in x days, weeks, or months.'
            status-fake:
                type: int
                description: 'Fake value for the menu to work.'

'''

EXAMPLES = '''
 - hosts: fortimanager-inventory
   collections:
     - fortinet.fortimanager
   connection: httpapi
   vars:
      ansible_httpapi_use_ssl: True
      ansible_httpapi_validate_certs: False
      ansible_httpapi_port: 443
   tasks:
    - name: Automatic deletion policy for logs, reports, archived, and quarantined files.
      fmgr_system_autodelete:
         bypass_validation: False
         workspace_locking_adom: <value in [global, custom adom including root]>
         workspace_locking_timeout: 300
         rc_succeeded: [0, -2, -3, ...]
         rc_failed: [-2, -3, ...]
         system_autodelete:
            dlp-files-auto-deletion:
               retention: <value in [days, weeks, months]>
               runat: <value of integer>
               status: <value in [disable, enable]>
               value: <value of integer>
            log-auto-deletion:
               retention: <value in [days, weeks, months]>
               runat: <value of integer>
               status: <value in [disable, enable]>
               value: <value of integer>
            quarantine-files-auto-deletion:
               retention: <value in [days, weeks, months]>
               runat: <value of integer>
               status: <value in [disable, enable]>
               value: <value of integer>
            report-auto-deletion:
               retention: <value in [days, weeks, months]>
               runat: <value of integer>
               status: <value in [disable, enable]>
               value: <value of integer>
            status-fake: <value of integer>

'''

RETURN = '''
request_url:
    description: The full url requested
    returned: always
    type: str
    sample: /sys/login/user
response_code:
    description: The status of api request
    returned: always
    type: int
    sample: 0
response_message:
    description: The descriptive message of the api response
    type: str
    returned: always
    sample: OK.

'''
from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.connection import Connection
from ansible_collections.fortinet.fortimanager.plugins.module_utils.napi import NAPIManager
from ansible_collections.fortinet.fortimanager.plugins.module_utils.napi import check_galaxy_version
from ansible_collections.fortinet.fortimanager.plugins.module_utils.napi import check_parameter_bypass


def main():
    jrpc_urls = [
        '/cli/global/system/auto-delete'
    ]

    perobject_jrpc_urls = [
        '/cli/global/system/auto-delete/{auto-delete}'
    ]

    url_params = []
    module_primary_key = None
    module_arg_spec = {
        'enable_log': {
            'type': 'bool',
            'required': False,
            'default': False
        },
        'proposed_method': {
            'type': 'str',
            'required': False,
            'choices': [
                'set',
                'update',
                'add'
            ]
        },
        'bypass_validation': {
            'type': 'bool',
            'required': False,
            'default': False
        },
        'workspace_locking_adom': {
            'type': 'str',
            'required': False
        },
        'workspace_locking_timeout': {
            'type': 'int',
            'required': False,
            'default': 300
        },
        'rc_succeeded': {
            'required': False,
            'type': 'list'
        },
        'rc_failed': {
            'required': False,
            'type': 'list'
        },
        'system_autodelete': {
            'required': False,
            'type': 'dict',
            'revision': {
                '6.0.0': True,
                '6.2.1': True,
                '6.2.3': True,
                '6.2.5': True,
                '6.4.0': True,
                '6.4.2': True,
                '6.4.5': True,
                '7.0.0': True
            },
            'options': {
                'dlp-files-auto-deletion': {
                    'required': False,
                    'type': 'dict',
                    'options': {
                        'retention': {
                            'required': False,
                            'revision': {
                                '6.0.0': True,
                                '6.2.1': True,
                                '6.2.3': True,
                                '6.2.5': True,
                                '6.4.0': True,
                                '6.4.2': True,
                                '6.4.5': True,
                                '7.0.0': True
                            },
                            'choices': [
                                'days',
                                'weeks',
                                'months'
                            ],
                            'type': 'str'
                        },
                        'runat': {
                            'required': False,
                            'revision': {
                                '6.0.0': True,
                                '6.2.1': True,
                                '6.2.3': True,
                                '6.2.5': True,
                                '6.4.0': True,
                                '6.4.2': True,
                                '6.4.5': True,
                                '7.0.0': True
                            },
                            'type': 'int'
                        },
                        'status': {
                            'required': False,
                            'revision': {
                                '6.0.0': True,
                                '6.2.1': True,
                                '6.2.3': True,
                                '6.2.5': True,
                                '6.4.0': True,
                                '6.4.2': True,
                                '6.4.5': True,
                                '7.0.0': True
                            },
                            'choices': [
                                'disable',
                                'enable'
                            ],
                            'type': 'str'
                        },
                        'value': {
                            'required': False,
                            'revision': {
                                '6.0.0': True,
                                '6.2.1': True,
                                '6.2.3': True,
                                '6.2.5': True,
                                '6.4.0': True,
                                '6.4.2': True,
                                '6.4.5': True,
                                '7.0.0': True
                            },
                            'type': 'int'
                        }
                    }
                },
                'log-auto-deletion': {
                    'required': False,
                    'type': 'dict',
                    'options': {
                        'retention': {
                            'required': False,
                            'revision': {
                                '6.0.0': True,
                                '6.2.1': True,
                                '6.2.3': True,
                                '6.2.5': True,
                                '6.4.0': True,
                                '6.4.2': True,
                                '6.4.5': True,
                                '7.0.0': True
                            },
                            'choices': [
                                'days',
                                'weeks',
                                'months'
                            ],
                            'type': 'str'
                        },
                        'runat': {
                            'required': False,
                            'revision': {
                                '6.0.0': True,
                                '6.2.1': True,
                                '6.2.3': True,
                                '6.2.5': True,
                                '6.4.0': True,
                                '6.4.2': True,
                                '6.4.5': True,
                                '7.0.0': True
                            },
                            'type': 'int'
                        },
                        'status': {
                            'required': False,
                            'revision': {
                                '6.0.0': True,
                                '6.2.1': True,
                                '6.2.3': True,
                                '6.2.5': True,
                                '6.4.0': True,
                                '6.4.2': True,
                                '6.4.5': True,
                                '7.0.0': True
                            },
                            'choices': [
                                'disable',
                                'enable'
                            ],
                            'type': 'str'
                        },
                        'value': {
                            'required': False,
                            'revision': {
                                '6.0.0': True,
                                '6.2.1': True,
                                '6.2.3': True,
                                '6.2.5': True,
                                '6.4.0': True,
                                '6.4.2': True,
                                '6.4.5': True,
                                '7.0.0': True
                            },
                            'type': 'int'
                        }
                    }
                },
                'quarantine-files-auto-deletion': {
                    'required': False,
                    'type': 'dict',
                    'options': {
                        'retention': {
                            'required': False,
                            'revision': {
                                '6.0.0': True,
                                '6.2.1': True,
                                '6.2.3': True,
                                '6.2.5': True,
                                '6.4.0': True,
                                '6.4.2': True,
                                '6.4.5': True,
                                '7.0.0': True
                            },
                            'choices': [
                                'days',
                                'weeks',
                                'months'
                            ],
                            'type': 'str'
                        },
                        'runat': {
                            'required': False,
                            'revision': {
                                '6.0.0': True,
                                '6.2.1': True,
                                '6.2.3': True,
                                '6.2.5': True,
                                '6.4.0': True,
                                '6.4.2': True,
                                '6.4.5': True,
                                '7.0.0': True
                            },
                            'type': 'int'
                        },
                        'status': {
                            'required': False,
                            'revision': {
                                '6.0.0': True,
                                '6.2.1': True,
                                '6.2.3': True,
                                '6.2.5': True,
                                '6.4.0': True,
                                '6.4.2': True,
                                '6.4.5': True,
                                '7.0.0': True
                            },
                            'choices': [
                                'disable',
                                'enable'
                            ],
                            'type': 'str'
                        },
                        'value': {
                            'required': False,
                            'revision': {
                                '6.0.0': True,
                                '6.2.1': True,
                                '6.2.3': True,
                                '6.2.5': True,
                                '6.4.0': True,
                                '6.4.2': True,
                                '6.4.5': True,
                                '7.0.0': True
                            },
                            'type': 'int'
                        }
                    }
                },
                'report-auto-deletion': {
                    'required': False,
                    'type': 'dict',
                    'options': {
                        'retention': {
                            'required': False,
                            'revision': {
                                '6.0.0': True,
                                '6.2.1': True,
                                '6.2.3': True,
                                '6.2.5': True,
                                '6.4.0': True,
                                '6.4.2': True,
                                '6.4.5': True,
                                '7.0.0': True
                            },
                            'choices': [
                                'days',
                                'weeks',
                                'months'
                            ],
                            'type': 'str'
                        },
                        'runat': {
                            'required': False,
                            'revision': {
                                '6.0.0': True,
                                '6.2.1': True,
                                '6.2.3': True,
                                '6.2.5': True,
                                '6.4.0': True,
                                '6.4.2': True,
                                '6.4.5': True,
                                '7.0.0': True
                            },
                            'type': 'int'
                        },
                        'status': {
                            'required': False,
                            'revision': {
                                '6.0.0': True,
                                '6.2.1': True,
                                '6.2.3': True,
                                '6.2.5': True,
                                '6.4.0': True,
                                '6.4.2': True,
                                '6.4.5': True,
                                '7.0.0': True
                            },
                            'choices': [
                                'disable',
                                'enable'
                            ],
                            'type': 'str'
                        },
                        'value': {
                            'required': False,
                            'revision': {
                                '6.0.0': True,
                                '6.2.1': True,
                                '6.2.3': True,
                                '6.2.5': True,
                                '6.4.0': True,
                                '6.4.2': True,
                                '6.4.5': True,
                                '7.0.0': True
                            },
                            'type': 'int'
                        }
                    }
                },
                'status-fake': {
                    'required': False,
                    'revision': {
                        '6.0.0': True,
                        '6.2.1': True,
                        '6.2.3': True,
                        '6.2.5': True,
                        '6.4.0': True,
                        '6.4.2': True,
                        '6.4.5': True,
                        '7.0.0': True
                    },
                    'type': 'int'
                }
            }

        }
    }

    params_validation_blob = []
    check_galaxy_version(module_arg_spec)
    module = AnsibleModule(argument_spec=check_parameter_bypass(module_arg_spec, 'system_autodelete'),
                           supports_check_mode=False)

    fmgr = None
    if module._socket_path:
        connection = Connection(module._socket_path)
        connection.set_option('enable_log', module.params['enable_log'] if 'enable_log' in module.params else False)
        fmgr = NAPIManager(jrpc_urls, perobject_jrpc_urls, module_primary_key, url_params, module, connection, top_level_schema_name='data')
        fmgr.validate_parameters(params_validation_blob)
        fmgr.process_partial_curd(argument_specs=module_arg_spec)
    else:
        module.fail_json(msg='MUST RUN IN HTTPAPI MODE')
    module.exit_json(meta=module.params)


if __name__ == '__main__':
    main()
